package info.jedi.testsuite;

import info.jedi.net.*;
import info.jedi.*;
import java.util.*;

        /** This class is designed to test the parts.basic package and several
         * classes that this package depends on.  It is run from the command
         * line with the following syntax:<p>
         * <pre>
         * java Client <server address> [<number of messages>]
         * </pre>
         * The address is that of a machine where the Server class
         * is running (run with java Server)
         * The number of messages is an optional argument for testing
         * reliability with many, many message sends (it has been tested
         * successfully with 1,000,000 messages as this argument).<p>
         *
         * Only one client can properly test a single server at a time; not
         * following this rule may result in test failure or even a VM crash.<p>
         *
         */

public class Client
{
                /** Creates a Client object that will test a server identified
                 * by its address and port and will send a certain number of
                 * simple messages in the process.
                 *
                 * @exception   Exception if there is trouble connecting
                 *              to the server.
                 */
	public Client(String serverAddress, int serverPort, int numMessages)
		throws Exception
	{
		this.numMessages = numMessages;
		server = new Proxy(serverAddress, serverPort, "JEDI Test Suite Server");
		serverClass = new Proxy(serverAddress, serverPort, "info.jedi.testsuite.Server");
		Repository.local().bind(this, "JEDI Test Suite Client");
		me = new Proxy("JEDI Test Suite Client");
	}

                /** Reports a boolean result on standard output.  It will
                 * print "success" if true and "failure" if false.  This
                 * method returns the passed boolean value.
                 */

	private boolean reportResult(boolean success)
	{
		if (success)
			System.out.println("success");
		else
			System.out.println("failure");

		return success;
	}
			
                /** Runs the basic communication tests that try to send global
                 * method calls with different parameters.
                 *      
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */
	public boolean runBaseTests()
		throws Exception
	{
		boolean success = true;

		System.out.println("Running base tests:");
		System.out.print("\tRunning integer test...");
		success &= reportResult(((Integer)server.function("intFunction", new Integer(5))).intValue() == -5);

		System.out.print("\tRunning boolean test...");
		success &= reportResult(((Boolean)server.function("booleanFunction", new Boolean(true))).booleanValue() == false);

		System.out.print("\tRunning String test...");
		success &= reportResult("UPPER".equals(server.function("stringFunction", "upper")));

		System.out.print("\tRunning ComplexObject test...");
		success &= reportResult((ComplexObject)server.function("complexObjectFunction", new ComplexObject(6,1)) != null);

		System.out.print("\tRunning multiple argument test...");
		Vector vector = new Vector();
		vector.addElement(new Integer(5));
		vector.addElement(new Boolean(true));
		vector.addElement(new Float(5.0));
		vector.addElement(new Double(5.0));
		vector.addElement("Hello");
		vector.addElement(new ComplexObject(6,1));
		success &= reportResult(((Vector)server.function("multipleFunction", vector)).size() == 6);

		System.out.print("\tRunning static method test...");
		success &= reportResult(((Integer)serverClass.function("staticFunction", new Integer(6))).intValue() == 60);

		return success;
	}

                /** Runs the semaphore tests, where 100 remote functions are
                 * called simultaneously, and all calls block until the 100th
                 * call has been received.
                 *
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */
	public boolean runSemaphoreTests()
	{
		System.out.print("Running semaphore tests...");

		for (int i = 1; i < Server.SEMAPHORE_NUMBER; ++i)
			new SemaphoreThread(server).start();
		SemaphoreThread last = new SemaphoreThread(server);
		last.run();

			// successful if it completes!
		System.out.println("success");
		return true;
	}

                /** Runs the counting tests.  The number of messages passed
                 * to the constructor of Client is sent to the server, and the
                 * server keeps a running total of the number of messages.
                 * When this running total is returned from every method call
                 * to the client the client checks it against its loop variables
                 * to make sure each call made was received exactly once.
                 *
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */     
	public boolean runCountTests()
		throws Exception
	{
		System.out.println("Running counting tests:");
		int batch = numMessages / 10;
		int count = 0;

		for (int i = 0; i < 10; ++i)
		{
			long start = System.currentTimeMillis();

			for (int j = 0; j < batch; ++j)
			{
				int value = ((Integer)server.function(
						"countFunction")).intValue();
				if (value != count++)
				{
					System.out.println("\tcount call returned " + value + "; expected " + (count-1));
					return false;
				}
			}

			long difference = System.currentTimeMillis() - start;

			System.out.println("\t" + batch + " messages sent with average round trip latency " + ((double)difference)/batch + " ms");
		}

		return true;
	}

                /** Recursively bounces a call between client and server,
                 * passing a Proxy representing this object and an ever
                 * decreasing index of how many recursive calls are left.
                 *                              
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */
        public void bounceFunction(Proxy next, int bouncesLeft)
		throws Exception
        {
                if (bouncesLeft-- > 0)
                        next.function("bounceFunction", me,
					new Integer(bouncesLeft));
        }

                /** Uses the bounceFunction to bounce a call back and forth
                 * between the client and server 100 times.
                 */
	public boolean runProxyTests()
	{
		System.out.print("Running Proxy (back and forth) tests...");

		try {

		server = (Proxy) server.function("getProxy");
		server.function("bounceFunction", me, new Integer(100));

		} catch (Exception e)
		{
			e.printStackTrace();
			System.out.println("failure");
			return false;
		}

		System.out.println("success");
		return true;
	}

                /** Runs tests designed to fail in different ways.  In addition
                 * to printing success anytime an exception was thrown when
                 * expected, this method prints the exception to standard out
                 * so that it can be visually checked for accuracy.
                 */
	public boolean runExceptionTests()
	{
		boolean success = true;
		System.out.println("Running exception tests:");
		System.out.print("\tRunning method throwing exception test...");
		try { server.function("throwsException"); success = false; }
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		System.out.print("\tRunning method that doesn't exist test...");
		try { server.function("doesNotExist"); success = false; }
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		System.out.print("\tRunning object that doesn't exist test...");
		try {
			Proxy p = new Proxy(server.machineName(), server.port(), "doesNotExist");
			p.function("hello");
			success = false;
		}
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		System.out.print("\tRunning unserializable parameter test...");
		try { server.function("unserializableParameter", this); success = false; }
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		System.out.print("\tRunning unserializable return value test...");
		try { server.function("unserializableReturnValue"); success = false; }
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		System.out.print("\tRunning server doesn't exist test...");
		try {
			Proxy p = new Proxy ("stun4w.cs", 9824, "hello");
			p.function("hello");
			success = false;
		}
		catch (Exception e) { System.out.println("success "); e.printStackTrace(); }

		return success;
	}

                /** Runs all the tests in the suite, printing success or
                 * failure after all tests have been run.
                 *
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */
	public boolean runTests()
		throws Exception
	{
		boolean success = true;

		System.out.print("Initializing...");
		server.function("initialize");
		System.out.println("success");

		success &= runBaseTests();

		success &= runCountTests();

		success &= runSemaphoreTests();

		success &= runProxyTests();

		success &= runExceptionTests();

		return success;
	}

                /** Handles command line invocations of the client side of the
                 * test suite.  See the class description for the usage.
                 */
	public static void main(String args[])
	{
		boolean success = false;

		try {
		int serverPort = MailDaemon.DEFAULT_PORT;
		if (args.length < 1)
		{
			System.out.println("usage: java Client <server address> [<number of messages>] [<server port>]");
			System.exit(-1);
		}

		String serverAddress = args[0];
		int numMessages = 1000;
		if (args.length > 1)
			numMessages = Integer.parseInt(args[1]);
		if (args.length > 2)
			serverPort = Integer.parseInt(args[2]);

		Client client = new Client(serverAddress, serverPort, numMessages);

		success = client.runTests();

		} catch (Exception e) { e.printStackTrace(); }

		if (success)
			System.out.println("All tests completed successfully");
		else
			System.out.println("Some tests failed");

		System.exit(0);
	}

                /** A proxy to the test server this client interacts with */
	private Proxy server;

                /** A proxy to the class parts.basic.testsuite.Server on the
                 * machine where the server is.  Used to call static methods
                 * on the server machine.
                 */
	private Proxy serverClass;

                /** The number of simple messages to send.
                 */
	private int numMessages;

                /** A proxy to this client */
	private Proxy me;
}

