package info.jedi.testsuite;

import info.jedi.*;
import info.jedi.net.*;
import java.util.*;
import java.net.*;

        /** Class Server helps to test the parts.basic package by providing a
         * set of methods that can be called remotely.  A Server is started
         * from the command line with the simple syntax<p><pre>
         *
         * java Server
         */
public class Server
{
                /** The number of calls necessary to the semaphore before it
                 * triggers and releases all threads inside.
                 */
	public static final int SEMAPHORE_NUMBER = 100;

                /** Creates a Server */
	public Server()
	{
		messageNumber = 0;
		numWaiting = 0;
	}

                /** Initializes the server.  Primarily this resets the internal
                 * count so that it is synchronized with the client's count.
                 * This count is later used in countFunction.
                 */
	public void initialize()
	{
		messageNumber = 0;
	}

                /** Used to test String arguments and return values.  Returns
                 * the passed string in upper case.
                 */
	public String stringFunction(String argument)
	{
		return argument.toUpperCase();
	}

                /** Used to test boolean arguments and return values.  Returns
                 * the inverse of the argument.
                 */
	public boolean booleanFunction(boolean argument)
	{
		return !argument;
	}

                /** Used to test integer arguments and return values.  Returns
                 * the negation of the argument.
                 */
	public int intFunction(int argument)
	{
		return -argument;
	}

                /** Used to test serializable complex object arguments and
                 * return values.  Touches the object and returns it.  See
                 * ComplexObject for a description of the object passed--it is
                 * a complicated tree-like structure.
                 */
	public ComplexObject complexObjectFunction(ComplexObject argument)
	{
		argument.touch();
		return argument;
	}

                /** Takes 6 arguments, all of different types: some object,
                 * some basic.  Returns a vector with copies of all the
                 * arguments, except that the complex object is just touched. 
                 */
	public Vector multipleFunction(int arg1, boolean arg2, float arg3,
		double arg4, String arg5, ComplexObject arg6)
	{
		Vector v = new Vector();
		v.addElement(new Integer(arg1));
		v.addElement(new Boolean(arg2));
		v.addElement(new Float(arg3));
		v.addElement(new Double(arg4));
		v.addElement(arg5);
		v.addElement(arg6);
		arg6.touch();
		return v;
	}

                /** For testing static functions.  Returns argument * 10. */
	public static int staticFunction(int argument)
	{
		return argument*10;
	}

                /** Increments the internal message count and returns the
                 * current count. */
	public int countFunction()
	{
		return messageNumber++;
	}

                /** Acts like a semaphore.  All incoming methods block until
                 * SEMAPHORE_NUMBER of method have been received.  Then all
                 * blocked processes are notified and return.
                 *
                 * @exception   InterruptedException if this thread is
                 *                      interrupted while waiting on the
                 *                      semaphore
                 */
	public synchronized void semaphoreFunction()
		throws InterruptedException
	{
		numWaiting++;
		if (numWaiting != SEMAPHORE_NUMBER)
			wait();
		else
		{
			notifyAll();
			numWaiting = 0;
		}
	}

                /** Returns a proxy to this Server */
	public Proxy getProxy()
		throws UnknownHostException
	{
		me = new Proxy("JEDI Test Suite Server");
		return me;
	}

                /** Bounces a call recursively between client and server,
                 * decrementing the number of bounces left at each
                 * call until they are 0, at which point the whole stack
                 * unwinds.  Each call passes the proxy that will be called
                 * next to the other machine, ensuring that Proxies can be
                 * passed reliably.
                 *
                 * @exception   Exception if one of the global method calls
                 *                      throws an unexpected exception
                 */
	public void bounceFunction(Proxy next, int bouncesLeft)
		throws Exception
	{
		if (bouncesLeft-- > 0)
			next.function("bounceFunction", me,
					new Integer(bouncesLeft));
	}

                /** A method that throws an exception */
	public int throwsException() throws SocketException
	{
		int i = 5;
		throw new SocketException("Didn't try to create a socket");
	}
	
                /** A method with an unserializable parameter. */
	public void unserializableParameter(Client client)
	{
	}

                /** A method that returns an unserializable value. */
	public Server unserializableReturnValue()
	{
		return this;
	}

		/** Creates a server.  An optional first argument will
		 * set the server up at a different port.
		 */
	public static void main(String args[])
		throws SocketException
	{
		Server server = new Server();

		if (args.length > 0)
			new MailDaemon(Integer.parseInt(args[0]));

		Repository.local().bind(server,"JEDI Test Suite Server");
	}

                /** The internal count of the nubmer of countFunction() calls
                 * since the last call to initialize()
                 */
	private int messageNumber;

                /** The nubmer of waiting threads in teh semaphore */
	private int numWaiting;

                /** A proxy to this server. */
	private Proxy me;
}

